<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2017 Wolf Utz <utz@riconet.de>, riconet
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

namespace Riconet\RicoProducts\Controller;

use TYPO3\CMS\Extbase\Domain\Model\Category;
use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;
use TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface;
use Riconet\RicoProducts\Domain\Model\Product;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * Class ProductController.
 */
class ProductController extends ActionController
{
    /**
     * productRepository.
     *
     * @var \Riconet\RicoProducts\Domain\Repository\ProductRepository
     * @inject
     */
    protected $productRepository = null;

    /**
     * categoryHelper.
     *
     * @var \Riconet\RicoProducts\Helper\CategoryHelper
     * @inject
     */
    protected $categoryHelper = null;

    /**
     * persistence.
     *
     * @var int
     */
    protected $persistence = [];

    /**
     * singlePid.
     *
     * @var int
     */
    protected $singlePid = 0;

    /**
     * recursions.
     *
     * @var int
     */
    protected $recursions = 0;

    /**
     * Action index.
     */
    public function initializeAction()
    {
        $configuration = $this->configurationManager->getConfiguration(
            ConfigurationManagerInterface::CONFIGURATION_TYPE_FRAMEWORK
        );
        $this->persistence = $configuration['persistence'];
        $this->singlePid = (int) $this->settings['singlePid'] > 0 ? (int) $this->settings['singlePid'] :
            (int) $this->persistence['singlePid'];
        if (isset($this->settings['recursions']) && $this->settings['recursions'] >= 0) {
            $this->recursions = (int) $this->settings['recursions'];
        }
    }

    /**
     * Action list.
     *
     * @param Category $category
     */
    public function listAction(Category $category = null)
    {
        $filter = $this->request->getArguments()['filter'];
        $limitCategories = $this->categoryHelper->getCategoriesByUids(
            GeneralUtility::intExplode(',', $this->settings['categoryLimit'])
        );
        if ($category instanceof Category) {
            $products = $this->productRepository->findByCategory($category, $this->recursions);
        } elseif (count($limitCategories) > 0) {
            $products = $this->productRepository->findByCategories($limitCategories);
        } else {
            $products = $this->productRepository->findAll();
        }
        $this->view->assignMultiple([
            'products' => $products,
            'singlePid' => $this->singlePid,
            'itemsPerPage' => $this->settings['itemsPerPage'],
            'insertAbove' => $this->settings['insertAbove'],
            'insertBelow' => $this->settings['insertBelow'],
            'maximumNumberOfLinks' => $this->settings['maximumNumberOfLinks'],
            'filter' => $filter,
        ]);
    }

    /**
     * Action show.
     *
     * @param \Riconet\RicoProducts\Domain\Model\Product $product
     */
    public function showAction(Product $product)
    {
        $fileCollections = null;
        if ($product->getFileCollections()) {
            $fileCollections = $this->categoryHelper->getFileCollections(
                GeneralUtility::intExplode(',', $product->getFileCollections())
            );
        }
        $this->view->assignMultiple([
            'product' => $product,
            'fileCollections' => $fileCollections,
        ]);
    }
}
