<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2017 Wolf Utz <utz@riconet.de>, riconet
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

namespace Riconet\RicoProducts\Domain\Model;

use TYPO3\CMS\Extbase\DomainObject\AbstractEntity;
use TYPO3\CMS\Extbase\Persistence\ObjectStorage;
use TYPO3\CMS\Extbase\Domain\Model\Category as T3Category;

/**
 * Class Product.
 */
class Product extends AbstractEntity
{
    use \Riconet\RicoProducts\Traits\SysLanguageTrait;

    /**
     * title.
     *
     * @var string
     * @validate NotEmpty
     */
    protected $title = '';

    /**
     * subtitle.
     *
     * @var string
     */
    protected $subtitle = '';

    /**
     * articleNumber.
     *
     * @var string
     * @validate NotEmpty
     */
    protected $articleNumber = '';

    /**
     * teaser.
     *
     * @var string
     */
    protected $teaser = '';

    /**
     * description.
     *
     * @var string
     */
    protected $description = '';

    /**
     * price.
     *
     * @var float
     */
    protected $price = 0.00;

    /**
     * images.
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\FileReference>
     * @cascade remove
     * @lazy
     */
    protected $images = null;

    /**
     * planImages.
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\FileReference>
     * @cascade remove
     * @lazy
     */
    protected $planImages = null;

    /**
     * dataSheets.
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\FileReference>
     * @cascade remove
     * @lazy
     */
    protected $dataSheets = null;

    /**
     * manuals.
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\FileReference>
     * @cascade remove
     * @lazy
     */
    protected $manuals = null;

    /**
     * files.
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\FileReference>
     * @cascade remove
     * @lazy
     */
    protected $files = null;

    /**
     * contents.
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\Content>
     * @cascade remove
     * @lazy
     */
    protected $contents = null;

    /**
     * attributes.
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\AttributeData>
     * @cascade remove
     * @lazy
     */
    protected $attributes = null;

    /**
     * prices.
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\Price>
     * @cascade remove
     * @lazy
     */
    protected $prices = null;

    /**
     * variants.
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\Variant>
     * @cascade remove
     * @lazy
     */
    protected $variants = null;

    /**
     * categories.
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\CMS\Extbase\Domain\Model\Category>
     * @lazy
     */
    protected $categories = null;

    /**
     * accessories.
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\Product>
     * @lazy
     */
    protected $accessories = null;

    /**
     * fileCollections.
     *
     * @var string
     */
    protected $fileCollections = '';

    /**
     * discounts.
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\Discount>
     * @lazy
     */
    protected $discounts = null;

    /**
     * __construct.
     */
    public function __construct()
    {
        $this->initStorageObjects();
    }

    /**
     * Initializes all ObjectStorage properties.
     */
    protected function initStorageObjects()
    {
        $this->images = new ObjectStorage();
        $this->planImages = new ObjectStorage();
        $this->files = new ObjectStorage();
        $this->dataSheets = new ObjectStorage();
        $this->manuals = new ObjectStorage();
        $this->contents = new ObjectStorage();
        $this->attributes = new ObjectStorage();
        $this->prices = new ObjectStorage();
        $this->variants = new ObjectStorage();
        $this->categories = new ObjectStorage();
        $this->accessories = new ObjectStorage();
        $this->discounts = new ObjectStorage();
    }

    /**
     * Returns the title.
     *
     * @return string $title
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * Sets the title.
     *
     * @param string $title
     */
    public function setTitle($title)
    {
        $this->title = $title;
    }

    /**
     * Returns the teaser.
     *
     * @return string $teaser
     */
    public function getTeaser()
    {
        return $this->teaser;
    }

    /**
     * Sets the teaser.
     *
     * @param string $teaser
     */
    public function setTeaser($teaser)
    {
        $this->teaser = $teaser;
    }

    /**
     * Returns the description.
     *
     * @return string $description
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * Sets the description.
     *
     * @param string $description
     */
    public function setDescription($description)
    {
        $this->description = $description;
    }

    /**
     * Returns the price.
     *
     * @return float $price
     */
    public function getPrice()
    {
        return $this->price;
    }

    /**
     * Sets the price.
     *
     * @param float $price
     */
    public function setPrice($price)
    {
        $this->price = $price;
    }

    /**
     * Adds a Image.
     *
     * @param \Riconet\RicoProducts\Domain\Model\FileReference $image
     */
    public function addImage(FileReference $image)
    {
        $this->images->attach($image);
    }

    /**
     * Removes a Image.
     *
     * @param \Riconet\RicoProducts\Domain\Model\FileReference $imageToRemove The Image to be removed
     */
    public function removeImage(FileReference $imageToRemove)
    {
        $this->images->detach($imageToRemove);
    }

    /**
     * Checks if images contains an image.
     *
     * @param string $fileName the file to look for
     *
     * @return bool
     */
    public function containsImage($fileName)
    {
        if (count($this->images) > 0) {
            foreach ($this->images as $file) {
                if ($fileName == $file->getOriginalResource()->getOriginalFile()->getName()) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Removes all Images.
     */
    public function removeAllImages()
    {
        if ($this->images) {
            $this->images->removeAll($this->images);
        }
    }

    /**
     * Returns the images.
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\FileReference> $images
     */
    public function getImages()
    {
        return $this->images;
    }

    /**
     * Sets the images.
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\FileReference> $images
     */
    public function setImages(ObjectStorage $images)
    {
        $this->images = $images;
    }

    /**
     * Adds a File.
     *
     * @param \Riconet\RicoProducts\Domain\Model\FileReference $file
     */
    public function addFile(FileReference $file)
    {
        $this->files->attach($file);
    }

    /**
     * Removes a File.
     *
     * @param \Riconet\RicoProducts\Domain\Model\FileReference $fileToRemove The File to be removed
     */
    public function removeFile(FileReference $fileToRemove)
    {
        $this->files->detach($fileToRemove);
    }

    /**
     * Checks if files contains a file.
     *
     * @param string $fileName the file to look for
     *
     * @return bool
     */
    public function containsFile($fileName)
    {
        if (count($this->files) > 0) {
            foreach ($this->files as $file) {
                if ($fileName == $file->getOriginalResource()->getOriginalFile()->getName()) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Removes all Files.
     */
    public function removeAllFiles()
    {
        if ($this->files) {
            $this->files->removeAll($this->files);
        }
    }

    /**
     * Returns the files.
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\FileReference> $files
     */
    public function getFiles()
    {
        return $this->files;
    }

    /**
     * Sets the files.
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\FileReference> $files
     */
    public function setFiles(ObjectStorage $files)
    {
        $this->files = $files;
    }

    /**
     * Returns the dataSheets.
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\FileReference>
     */
    public function getDataSheets()
    {
        return $this->dataSheets;
    }

    /**
     * Sets the files.
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\FileReference> $dataSheets
     */
    public function setDataSheets(ObjectStorage $dataSheets)
    {
        $this->dataSheets = $dataSheets;
    }

    /**
     * Returns the manuals.
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\FileReference>
     */
    public function getManuals()
    {
        return $this->manuals;
    }

    /**
     * Sets the manuals.
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\FileReference> $manuals
     */
    public function setManuals(ObjectStorage $manuals)
    {
        $this->manuals = $manuals;
    }

    /**
     * Adds a Planimage.
     *
     * @param \Riconet\RicoProducts\Domain\Model\FileReference $planImage
     */
    public function addPlanimage(FileReference $planImage)
    {
        $this->planImages->attach($planImage);
    }

    /**
     * Removes a Planimage.
     *
     * @param \Riconet\RicoProducts\Domain\Model\FileReference $planImageToRemove The Planimage to be removed
     */
    public function removePlanimage(FileReference $planImageToRemove)
    {
        $this->planImages->detach($planImageToRemove);
    }

    /**
     * Checks if planimages contains an planimage.
     *
     * @param string $fileName the file to look for
     *
     * @return bool
     */
    public function containsPlanimage($fileName)
    {
        if (count($this->planImages) > 0) {
            foreach ($this->planImages as $file) {
                if ($fileName == $file->getOriginalResource()->getOriginalFile()->getName()) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Removes all Planimages.
     */
    public function removeAllPlanimages()
    {
        if ($this->planImages) {
            $this->planImages->removeAll($this->planImages);
        }
    }

    /**
     * Returns the planimages.
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\FileReference> $planimages
     */
    public function getPlanimages()
    {
        return $this->planImages;
    }

    /**
     * Sets the planimages.
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\FileReference> $planimages
     */
    public function setPlanimages(ObjectStorage $planimages)
    {
        $this->planImages = $planimages;
    }

    /**
     * Adds a Variant.
     *
     * @param \Riconet\RicoProducts\Domain\Model\Variant $variant
     */
    public function addVariant(Variant $variant)
    {
        $this->variants->attach($variant);
    }

    /**
     * Removes a Variant.
     *
     * @param \Riconet\RicoProducts\Domain\Model\Variant $variantToRemove The Variant to be removed
     */
    public function removeVariant(Variant $variantToRemove)
    {
        $this->variants->detach($variantToRemove);
    }

    /**
     * Removes all Variants.
     */
    public function removeAllVariants()
    {
        if ($this->variants) {
            $this->variants->removeAll($this->variants);
        }
    }

    /**
     * Returns the variants.
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\Variant> $variants
     */
    public function getVariants()
    {
        return $this->variants;
    }

    /**
     * Sets the variants.
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\Variant> $variants
     */
    public function setVariants(ObjectStorage $variants)
    {
        $this->variants = $variants;
    }

    /**
     * Adds a Product.
     *
     * @param \Riconet\RicoProducts\Domain\Model\Product $accessory
     */
    public function addAccessory(Product $accessory)
    {
        $this->accessories->attach($accessory);
    }

    /**
     * Removes a Product.
     *
     * @param \Riconet\RicoProducts\Domain\Model\Product $accessoryToRemove The Product to be removed
     */
    public function removeAccessory(Product $accessoryToRemove)
    {
        $this->accessories->detach($accessoryToRemove);
    }

    /**
     * Removes all Accessories.
     */
    public function removeAllAccessories()
    {
        if ($this->accessories) {
            $this->accessories->removeAll($this->accessories);
        }
    }

    /**
     * Returns the accessories.
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\Product> $accessories
     */
    public function getAccessories()
    {
        return $this->accessories;
    }

    /**
     * Sets the accessories.
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\Product> $accessories
     */
    public function setAccessories(ObjectStorage $accessories)
    {
        $this->accessories = $accessories;
    }

    /**
     * Adds a Attribute.
     *
     * @param \Riconet\RicoProducts\Domain\Model\AttributeData $attribute
     */
    public function addAttribute(AttributeData $attribute)
    {
        $this->attributes->attach($attribute);
    }

    /**
     * Removes a Attribute.
     *
     * @param \Riconet\RicoProducts\Domain\Model\AttributeData $attributeToRemove The Attribute to be removed
     */
    public function removeAttribute(AttributeData $attributeToRemove)
    {
        $this->attributes->detach($attributeToRemove);
    }

    /**
     * Removes all Attributes.
     */
    public function removeAllAttributes()
    {
        if ($this->attributes) {
            $this->attributes->removeAll($this->attributes);
        }
    }

    /**
     * Returns the attributes.
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\AttributeData> $attributes
     */
    public function getAttributes()
    {
        return $this->attributes;
    }

    /**
     * Sets the attributes.
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\AttributeData> $attributes
     */
    public function setAttributes(ObjectStorage $attributes)
    {
        $this->attributes = $attributes;
    }

    /**
     * Returns the subtitle.
     *
     * @return string $subtitle
     */
    public function getSubtitle()
    {
        return $this->subtitle;
    }

    /**
     * Sets the subtitle.
     *
     * @param string $subtitle
     */
    public function setSubtitle($subtitle)
    {
        $this->subtitle = $subtitle;
    }

    /**
     * Returns the articleNumber.
     *
     * @return string $articleNumber
     */
    public function getArticleNumber()
    {
        return $this->articleNumber;
    }

    /**
     * Sets the articleNumber.
     *
     * @param string $articleNumber
     */
    public function setArticleNumber($articleNumber)
    {
        $this->articleNumber = $articleNumber;
    }

    /**
     * Adds a Content.
     *
     * @param \Riconet\RicoProducts\Domain\Model\Content $content
     */
    public function addContent(Content $content)
    {
        $this->contents->attach($content);
    }

    /**
     * Removes a Content.
     *
     * @param \Riconet\RicoProducts\Domain\Model\Content $contentToRemove The Content to be removed
     */
    public function removeContent(Content $contentToRemove)
    {
        $this->contents->detach($contentToRemove);
    }

    /**
     * Removes all Contents.
     */
    public function removeAllContents()
    {
        if ($this->contents) {
            $this->contents->removeAll($this->contents);
        }
    }

    /**
     * Returns the contents.
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\Content> $contents
     */
    public function getContents()
    {
        return $this->contents;
    }

    /**
     * Sets the contents.
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\Content> $contents
     */
    public function setContents(ObjectStorage $contents)
    {
        $this->contents = $contents;
    }

    /**
     * Adds a Category.
     *
     * @param \TYPO3\CMS\Extbase\Domain\Model\Category $category
     */
    public function addCategory(T3Category $category)
    {
        $this->categories->attach($category);
    }

    /**
     * Removes a Category.
     *
     * @param \TYPO3\CMS\Extbase\Domain\Model\Category $categoryToRemove The Category to be removed
     */
    public function removeCategory(T3Category $categoryToRemove)
    {
        $this->categories->detach($categoryToRemove);
    }

    /**
     * Removes all Categories.
     */
    public function removeAllCategories()
    {
        if ($this->categories) {
            $this->categories->removeAll($this->categories);
        }
    }

    /**
     * Returns the categories.
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\CMS\Extbase\Domain\Model\Category> $categories
     */
    public function getCategories()
    {
        return $this->categories;
    }

    /**
     * Sets the categories.
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\CMS\Extbase\Domain\Model\Category> $categories
     */
    public function setCategories(ObjectStorage $categories)
    {
        $this->categories = $categories;
    }

    /**
     * Adds a Price.
     *
     * @param \Riconet\RicoProducts\Domain\Model\Price $price
     */
    public function addPrice(Price $price)
    {
        $this->prices->attach($price);
    }

    /**
     * Removes a Price.
     *
     * @param \Riconet\RicoProducts\Domain\Model\Price $priceToRemove The Price to be removed
     */
    public function removePrice(Price $priceToRemove)
    {
        $this->prices->detach($priceToRemove);
    }

    /**
     * Returns the prices.
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\Price> $prices
     */
    public function getPrices()
    {
        return $this->prices;
    }

    /**
     * Sets the prices.
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\Price> $prices
     */
    public function setPrices(ObjectStorage $prices)
    {
        $this->prices = $prices;
    }

    /**
     * Returns the fileCollections.
     *
     * @return string $fileCollections
     */
    public function getFileCollections()
    {
        return $this->fileCollections;
    }

    /**
     * Sets the fileCollections.
     *
     * @param string $fileCollections
     */
    public function setFileCollections($fileCollections)
    {
        $this->fileCollections = $fileCollections;
    }

    /**
     * Returns the discounts.
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\Discount> $discounts
     */
    public function getDiscounts()
    {
        return $this->discounts;
    }

    /**
     * Sets the discounts.
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\Discount> $discounts
     */
    public function setDiscounts(ObjectStorage $discounts)
    {
        $this->discounts = $discounts;
    }

    /**
     * Adds a Discount.
     *
     * @param \Riconet\RicoProducts\Domain\Model\Discount $discount
     */
    public function addDiscount(Discount $discount)
    {
        $this->discounts->attach($discount);
    }

    /**
     * Removes a Discount.
     *
     * @param \Riconet\RicoProducts\Domain\Model\Discount $discountToRemove The Discount to be removed
     */
    public function removeDiscount(Discount $discountToRemove)
    {
        $this->discounts->detach($discountToRemove);
    }
}
