<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2017 Wolf Utz <utz@riconet.de>, riconet
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

namespace Riconet\RicoProducts\Domain\Model;

use TYPO3\CMS\Extbase\DomainObject\AbstractEntity;
use TYPO3\CMS\Extbase\Persistence\ObjectStorage;

/**
 * Class Variant.
 */
class Variant extends AbstractEntity
{
    use \Riconet\RicoProducts\Traits\SysLanguageTrait;

    /**
     * title.
     *
     * @var string
     */
    protected $title = '';

    /**
     * articleNumber.
     *
     * @var string
     * @validate NotEmpty
     */
    protected $articleNumber = 0;

    /**
     * subtitle.
     *
     * @var string
     */
    protected $subtitle = '';

    /**
     * description.
     *
     * @var string
     */
    protected $description = '';

    /**
     * price.
     *
     * @var float
     */
    protected $price = 0.00;

    /**
     * images.
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\FileReference>
     * @cascade remove
     * @lazy
     */
    protected $images = null;

    /**
     * prices.
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\Price>
     * @cascade remove
     * @lazy
     */
    protected $prices = null;

    /**
     * attributes.
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\AttributeData>
     * @cascade remove
     * @lazy
     */
    protected $attributes = null;

    /**
     * @var \Riconet\RicoProducts\Domain\Model\Product
     * @lazy
     */
    protected $product = null;

    /**
     * __construct.
     */
    public function __construct()
    {
        $this->initStorageObjects();
    }

    /**
     * Initializes all ObjectStorage properties.
     */
    protected function initStorageObjects()
    {
        $this->prices = new ObjectStorage();
        $this->attributes = new ObjectStorage();
        $this->images = new ObjectStorage();
    }

    /**
     * Returns the title.
     *
     * @return string $title
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * Sets the title.
     *
     * @param string $title
     */
    public function setTitle($title)
    {
        $this->title = $title;
    }

    /**
     * Returns the articleNumber.
     *
     * @return string $articleNumber
     */
    public function getArticleNumber()
    {
        return $this->articleNumber;
    }

    /**
     * Sets the articleNumber.
     *
     * @param string $articleNumber
     */
    public function setArticleNumber($articleNumber)
    {
        $this->articleNumber = $articleNumber;
    }

    /**
     * Returns the subtitle.
     *
     * @return string $subtitle
     */
    public function getSubtitle()
    {
        return $this->subtitle;
    }

    /**
     * Sets the subtitle.
     *
     * @param string $subtitle
     */
    public function setSubtitle($subtitle)
    {
        $this->subtitle = $subtitle;
    }

    /**
     * Returns the description.
     *
     * @return string $description
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * Sets the description.
     *
     * @param string $description
     */
    public function setDescription($description)
    {
        $this->description = $description;
    }

    /**
     * Returns the price.
     *
     * @return float $price
     */
    public function getPrice()
    {
        return $this->price;
    }

    /**
     * Sets the price.
     *
     * @param float $price
     */
    public function setPrice($price)
    {
        $this->price = $price;
    }

    /**
     * Adds a Price.
     *
     * @param \Riconet\RicoProducts\Domain\Model\Price $price
     */
    public function addPrice(Price $price)
    {
        $this->prices->attach($price);
    }

    /**
     * Removes a Price.
     *
     * @param \Riconet\RicoProducts\Domain\Model\Price $priceToRemove The Price to be removed
     */
    public function removePrice(Price $priceToRemove)
    {
        $this->prices->detach($priceToRemove);
    }

    /**
     * Returns the prices.
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\Price> $prices
     */
    public function getPrices()
    {
        return $this->prices;
    }

    /**
     * Sets the prices.
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\Price> $prices
     */
    public function setPrices(ObjectStorage $prices)
    {
        $this->prices = $prices;
    }

    /**
     * Adds a Attribute.
     *
     * @param \Riconet\RicoProducts\Domain\Model\AttributeData $attribute
     */
    public function addAttribute(AttributeData $attribute)
    {
        $this->attributes->attach($attribute);
    }

    /**
     * Removes a Attribute.
     *
     * @param \Riconet\RicoProducts\Domain\Model\AttributeData $attributeToRemove The Attribute to be removed
     */
    public function removeAttribute(AttributeData $attributeToRemove)
    {
        $this->attributes->detach($attributeToRemove);
    }

    /**
     * Removes all Attributes.
     */
    public function removeAllAttributes()
    {
        if ($this->attributes) {
            $this->attributes->removeAll($this->attributes);
        }
    }

    /**
     * Returns the attributes.
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\AttributeData> $attributes
     */
    public function getAttributes()
    {
        return $this->attributes;
    }

    /**
     * Sets the attributes.
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\AttributeData> $attributes
     */
    public function setAttributes(ObjectStorage $attributes)
    {
        $this->attributes = $attributes;
    }

    /**
     * Adds a Image.
     *
     * @param \Riconet\RicoProducts\Domain\Model\FileReference $image
     */
    public function addImage(FileReference $image)
    {
        $this->images->attach($image);
    }

    /**
     * Removes a Image.
     *
     * @param \Riconet\RicoProducts\Domain\Model\FileReference $imageToRemove The Image to be removed
     */
    public function removeImage(FileReference $imageToRemove)
    {
        $this->images->detach($imageToRemove);
    }

    /**
     * Checks if images contains an image.
     *
     * @param string $fileName the file to look for
     *
     * @return bool
     */
    public function containsImage($fileName)
    {
        if (count($this->images) > 0) {
            foreach ($this->images as $file) {
                if ($fileName == $file->getOriginalResource()->getOriginalFile()->getName()) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Removes all Images.
     */
    public function removeAllImages()
    {
        if ($this->images) {
            $this->images->removeAll($this->images);
        }
    }

    /**
     * Returns the images.
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\FileReference> $images
     */
    public function getImages()
    {
        return $this->images;
    }

    /**
     * Sets the images.
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Riconet\RicoProducts\Domain\Model\FileReference> $images
     */
    public function setImages(ObjectStorage $images)
    {
        $this->images = $images;
    }

    /**
     * @return \Riconet\RicoProducts\Domain\Model\Product
     */
    public function getProduct()
    {
        return $this->product;
    }

    /**
     * @param \Riconet\RicoProducts\Domain\Model\Product $product
     */
    public function setProduct(Product $product)
    {
        $this->product = $product;
    }
}
