<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2017 Wolf Utz <utz@riconet.de>, riconet
 *      Created on: 21.06.2017 16:05
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

namespace Riconet\RicoProducts\UserFunc;

use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Extbase\Utility\LocalizationUtility;

/**
 * Class TCATitleUserFunc.
 */
class TCATitleUserFunc
{
    /**
     * Formats the street label.
     *
     * @param array $parameters
     */
    public function formatPriceLabel(array &$parameters)
    {
        // Make sure to get the right record... do not know why we get more calls than records exist.
        if (!isset($parameters['row']['pid'])) {
            return;
        }
        $groupTitle = self::getUserGroupTitle((int) $parameters['row']['frontend_user_group']);
        $label = LocalizationUtility::translate(
            'LLL:EXT:rico_products/Resources/Private/Language/locallang_db.xlf:tx_ricoproducts_tca.prices_label',
            'RicoProducts',
            [!empty($groupTitle) ? $groupTitle : '---']
        );
        $parameters['title'] = $label;
    }

    /**
     * Formats the street label.
     *
     * @param array $parameters
     */
    public function formatScaledPriceLabel(array &$parameters)
    {
        $record = $parameters['row'];
        $label = LocalizationUtility::translate(
            'LLL:EXT:rico_products/Resources/Private/Language/locallang_db.xlf:tx_ricoproducts_tca.scaled_price_label',
            'RicoProducts',
            [$record['quantity'], $record['fixed_price'], $record['percentage_discount']]
        );
        $parameters['title'] = $label;
    }

    /**
     * Get the title of a user group.
     *
     * @param int $uid
     *
     * @return string
     */
    protected function getUserGroupTitle($uid)
    {
        if (!($uid > 0)) {
            return '';
        }
        $record = BackendUtility::getRecord('fe_groups', $uid);

        return isset($record['title']) ? $record['title'] : '';
    }
}
