<?php

namespace Riconet\RicoProducts\Utility;

/***************************************************************
 *
 *  Copyright notice
 *
 *  (c) 2016 Wolf Utz <utz@riconet.de>, riconet
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use Riconet\RicoProducts\Utility\ParseCSV;
use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;
use TYPO3\CMS\Extbase\Utility\LocalizationUtility;

/**
 * This class handles the import process.
 */
class ImportUtility
{
    /**
     * Makes sure that all needed fields are avqalable in the csv object.
     * 
     * @param type $allowedFields
     * @param \Riconet\RicoProducts\Utility\ParseCSV $csv
     * 
     * @return boolean
     */
    public static function validateFields(array $allowedFields, ParseCSV $csv)
    {
        return $allowedFields === array_intersect($allowedFields, $csv->titles);
    }
    
    /**
     * Creates an csv object by the given file.
     * 
     * @param array $file
     * 
     * @return \Riconet\RicoProducts\Utility\ParseCSV
     */
    public static function createCSVObject(array $file)
    {
        $path = ExtensionManagementUtility::extPath('rico_products').'tmp/';
        $file_path = $path.uniqid().'-'.basename($file['name']);
        move_uploaded_file($file['tmp_name'], $file_path);
        $csv = new ParseCSV();
        //$csv->auto($file_path);
        //$csv->encoding('UTF-8');
        $csv->delimiter = ";";
        $csv->parse($file_path);
        unlink($file_path);
        return $csv;
    }
    
    /**
     * Generates the option array for the import type select.
     *
     * @return array
     */
    public static function generateImportSelectOptions()
    {
        // Set the possible options for the select.
        $possible_options = array(
            LocalizationUtility::translate('tx_ricoproducts_module_import.option_products', 'RicoProducts'),
            LocalizationUtility::translate('tx_ricoproducts_module_import.option_attributes', 'RicoProducts'),
            LocalizationUtility::translate('tx_ricoproducts_module_import.option_categories', 'RicoProducts')
        );
        $options = array();
        for ($i = 0; $i < count($possible_options); $i ++) {
            $obj = new \stdClass();
            $obj->key = $i;
            $obj->value = $possible_options[$i];
            $options[] = $obj;
        }
        
        return $options;
    }
    
    /**
     * Splits a string into array items.
     * 
     * @param string $string
     * 
     * @return array
     */
    public static function splitToArray($string)
    {
        if(strpos($string, ',') === false) {
            return array($string);
        } else {
            $result = explode(',', $string);
            // Clean up...
            for($i = 0; $i < count($result); $i++) {
                $result[$i] = trim($result[$i]);
            }
            return $result;
        }
    }
    
    /**
     * Gets a path of an array.
     * 
     * @param int $from
     * @param int $till
     * @param array $array
     * 
     * @return array
     */
    public static function getArrayIndexFromTill($from, $till, array $array)
    {
        $newArray = array();
        if ($till <= count($array) && $till <= $till) {
            for ($i = $from; $i <= $till; $i++) {
                if (isset($array[$i])) {
                    $newArray[] = $array[$i];
                }
            }
            return $newArray;
        } else {
            return false;
        }
    }
}
