<?php

namespace Riconet\RicoProducts\ViewHelpers\BackendButtons;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2016 Wolf Utz <utz@riconet.de>, Riconet
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use TYPO3\CMS\Fluid\Core\ViewHelper\AbstractViewHelper;
use TYPO3\CMS\Core\FormProtection\FormProtectionFactory;
use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Extbase\DomainObject\AbstractEntity;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface;

/**
 * This ViewHelper creates a link to edit a record,
 * in a bakend module.
 *
 * This will only work in Typo3 V 7.5 + !!!
 *
 * How to use:
 *
 * {namespace r=Riconet\RicoProducts\ViewHelpers}
 *
 * <r:backendButtons.new table="tx_xxx_domain_model_xxx" pid="12" icon="actions-add" title="new record" />
 */
class NewViewHelper extends AbstractViewHelper
{
    /**
     * @var string $iconSrc
     */
    protected $iconSrc = '/typo3/sysext/core/Resources/Public/Icons/T3Icons/actions/';

    /**
     * Render the ViewHelper.
     *
     * @since 1.0.0
     *
     * @param string $table
     * @param string $icon
     * @param string $value
     * @param string $title
     * @param int $pid
     * @param bool $getPidByConf
     *
     * @return string The rendered viewhelper.
     */
    public function render($table, $icon, $value = '', $title = '', $pid = 0, $getPidByConf = false)
    {
        // Make sure we are in BE mode.
        if (TYPO3_MODE === 'BE') {
            if ($getPidByConf) {
                // Get the pid from confs.
                $confManager = GeneralUtility::makeInstance('TYPO3\\CMS\\Extbase\\Configuration\\ConfigurationManager');
                $conf = $confManager->getConfiguration(ConfigurationManagerInterface::CONFIGURATION_TYPE_FRAMEWORK);
                $pid = intval($conf['persistence']['storagePid']);
            }
            // Create the output.
            $html = '<a href="'.$this->generateUrlByTableAndPid($table, $pid).'" title="'.$title.'" class="btn btn-default">';
            $html .= '<span class="t3js-icon icon icon-size-small icon-state-default icon-actions-open" data-identifier="actions-open">';
            $html .= '<span class="icon-markup">';
            $html .= '<img width="16" height="16" src="'.$this->iconSrc.$icon.'.svg" />';
            $html .= '</span></span>';
            if (!empty($value)) {
                $html.= ' '.$value;
            }
            $html .= '</a>';
            
            return $html;
        } else {
            throw new \Exception("This NewViewHelper must only used in backend context!", 1);
        }
    }

    /**
     * Generates the edit record URL by a given table and pid.
     *
     * @since 1.0.0
     *
     * @param string $table
     * @param int $pid
     *
     * @return string The edit record url of the entity.
     */
    private function generateUrlByTableAndPid($table, $pid)
    {
        // Get the request.
        $request = $this->controllerContext->getRequest();
        // Get the plugin/module name.
        $pluginName = $request->getPluginName();
        // Generate the Token.
        $token = FormProtectionFactory::get()->generateToken('moduleCall', $pluginName);
        // Create the path. @TODO: figure out what this really is!
        $returnController = '&tx_'. strtolower($request->getControllerExtensionName()).'_'.strtolower($pluginName).'[controller]='.$request->getControllerName();
        // Generate the return url.
        $returnUrl = 'index.php?M='.$pluginName.'&id='.$pid.'&moduleToken='.$token.$returnController;
        
        return BackendUtility::getModuleUrl('record_edit', array(
            'edit['.$table.'][' . $pid . ']' => 'new',
            'returnUrl' => $returnUrl
        ));
    }

    /**
     * Generates a table name by a given AbstractEntity.
     *
     * @since 1.0.0
     *
     * @param \TYPO3\CMS\Extbase\DomainObject\AbstractEntity $entity
     *
     * @return string The table name of the entity.
     */
    private function classNameToTableName(AbstractEntity $entity)
    {
        // Get the class name.
        $className = \get_class($entity);
        // Remove the vendor path.
        $classPaths = explode('\\', strtolower($className));
        unset($classPaths[0]);

        return 'tx_'.implode('_', $classPaths);
    }
}
