<?php

namespace Riconet\RicoProducts\ViewHelpers\BackendLinks;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2016 Wolf Utz <utz@riconet.de>, Riconet
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use TYPO3\CMS\Fluid\Core\ViewHelper\AbstractTagBasedViewHelper;
use TYPO3\CMS\Core\FormProtection\FormProtectionFactory;
use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Extbase\DomainObject\AbstractEntity;

/**
 * This ViewHelper creates a link to edit a record,
 * in a bakend module.
 *
 * This will only work in Typo3 V 7.5 + !!!
 *
 * How to use:
 *
 * {namespace r=Riconet\RicoProducts\ViewHelpers}
 *
 * <r:backendLinks.edit entity="{product}">Edit</r:backendLinks.edit>
 *
 */
class EditViewHelper extends AbstractTagBasedViewHelper
{
    /**
     * @var string $tagName
     */
    protected $tagName = 'a';
    
    /**
     * Initialize arguments
     *
     * @since 1.0.0
     */
    public function initializeArguments()
    {
        $this->registerUniversalTagAttributes();
        $this->registerTagAttribute('name', 'string', 'Specifies the name of an anchor');
        $this->registerTagAttribute('target', 'string', 'Specifies where to open the linked document');
    }

    /**
     * Render the ViewHelper.
     *
     * @since 1.0.0
     *
     * @param \TYPO3\CMS\Extbase\DomainObject\AbstractEntity $entity
     *
     * @return string The rendered viewhelper.
     */
    public function render(AbstractEntity $entity)
    {
        // Make sure we are in BE mode.
        if (TYPO3_MODE === 'BE') {
            $this->tag->addAttribute('href', $this->generateUrlByEntity($entity));
            $this->tag->setContent($this->renderChildren());
            $this->tag->forceClosingTag(true);
            
            return $this->tag->render();
        } else {
            throw new \Exception("This NewViewHelper can must used in backend context!", 1);
        }
    }

    /**
     * Generates the edit record URL by a given AbstractEntity.
     *
     * @since 1.0.0
     *
     * @param \TYPO3\CMS\Extbase\DomainObject\AbstractEntity $entity
     *
     * @return string The edit record url of the entity.
     */
    private function generateUrlByEntity(AbstractEntity $entity)
    {
        // Get the tbale of the entity.
        $table = $this->classNameToTableName($entity);
        // Get the uid of the entity.
        $uid = $entity->getUid();
        // Get the plugin/module name.
        
        $pluginName = 'web_RicoProductsManagement';//$this->controllerContext->getRequest()->getPluginName();
        // Generate the Token.
        $token = FormProtectionFactory::get()->generateToken('moduleCall', $pluginName);
        // Generate the return url.
        $returnUrl = 'index.php?M='.$pluginName.'&id='.$uid.'&moduleToken='.$token;

        return BackendUtility::getModuleUrl('record_edit', array(
            'edit['.$table.'][' . $uid . ']' => 'edit',
            'returnUrl' => $returnUrl
        ));
    }

    /**
     * Generates a table name by a given AbstractEntity.
     *
     * @since 1.0.0
     *
     * @param \TYPO3\CMS\Extbase\DomainObject\AbstractEntity $entity
     *
     * @return string The table name of the entity.
     */
    private function classNameToTableName(AbstractEntity $entity)
    {
        // Get the class name.
        $className = \get_class($entity);
        // Remove the vendor path.
        $classPaths = explode('\\', strtolower($className));
        unset($classPaths[0]);

        return 'tx_'.implode('_', $classPaths);
    }
}
