<?php

namespace Riconet\RicoProducts\ViewHelpers;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2016 Wolf Utz <utz@riconet.de>, Riconet
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use TYPO3\CMS\Fluid\Core\ViewHelper\AbstractViewHelper;
use TYPO3\CMS\Fluid\Core\ViewHelper\Exception;

/**
 * Description of ParentToChildViewHelper
 *
 * @author utz
 */
class ParentToChildViewHelper extends AbstractViewHelper
{
    /**
     * The property name of the parent field.
     * 
     * @var string $parentFieldName
     */
    protected $parentFieldName = '';
    
    /**
     * Iterates through elements of $each and renders child nodes.
     *
     * @param array  $each The array or \TYPO3\CMS\Extbase\Persistence\ObjectStorage to iterated over
     * @param string $as The name of the iteration variable
     * @param string $parentFieldName
     * @param bool $isMM
     * 
     * @return string Rendered string
     *
     * @throws \TYPO3\CMS\Fluid\Core\ViewHelper\Exception
     */
    public function render($each, $as, $parentFieldName = 'parents', $isMM = false)
    {
        $output = '';
        if ($each === null) {
            return '';
        }
        // Check if the input each is traversable.
        if (is_object($each)) {
            if (!$each instanceof \Traversable) {
                throw new Exception('ParentToChildViewHelper only supports arrays and objects implementing \Traversable interface', 1253108907);
            }
            $each = iterator_to_array($each);
        }
        
        $this->parentFieldName = ucfirst($parentFieldName);
        
        // Assign view vars.
        foreach ($this->parentsToChildsOfParents($each) as $currentIndex => $index) {
            $this->templateVariableContainer->add($as, $index);
            $output .= $this->renderChildren();
            $this->templateVariableContainer->remove($as);
        }
        // Return the output.
        return $output;
    }
    
    /**
     * ...
     * 
     * @param array $items
     * 
     * @return array
     */
    private function parentsToChildsOfParents(array $items)
    {
        $childs = $this->getChilds($items);
        $parents = $this->getParents($items);
        return $this->createNewChildRelation($parents, $childs);
    }
    
    /**
     * ...
     * 
     * @param array $items
     * 
     * @return array
     * 
     * @throws \TYPO3\CMS\Fluid\Core\ViewHelper\Exception
     */
    private function getParents(array $items)
    {
        $parents = array();
        foreach ($items as $item) {
            if (method_exists($item, 'get'.$this->parentFieldName)) {
                if (count($item->{'get'.$this->parentFieldName}()) == 0) {
                    $parents[] = $item;
                }
            } else {
                throw new Exception('method '.'get'.$this->parentFieldName.' dont exist', 1253108907);
            }
        }
        return $parents;
    }
    
    /**
     * ...
     * 
     * @param array $items
     * 
     * @return array
     * 
     * @throws \TYPO3\CMS\Fluid\Core\ViewHelper\Exception
     */
    private function getChilds(array $items)
    {
        $childs = array();
        foreach ($items as $item) {
            if (method_exists($item, 'get'.$this->parentFieldName)) {
                if (count($item->{'get'.$this->parentFieldName}()) > 0) {
                    $childs[] = $item;
                }
            } else {
                throw new Exception('method '.'get'.$this->parentFieldName.' dont exist', 1253108907);
            }
        }
        return $childs;
    }
    
    /**
     * ...
     * 
     * @param array $parents
     * @param array $childs
     * 
     * @return array
     */
    private function createNewChildRelation(array $parents, array $childs)
    {
        for ($p = 0; $p < count($parents); $p++) {
            for ($c = 0; $c < count($childs); $c++) {
                $childsParent = $childs[$c]->{'get'.$this->parentFieldName}();
                if ($childsParent == $parents[$p]) {
                    $parents[$p]->{'set'.$this->parentFieldName}($childs[$c]);
                }
            }
        }
        return $parents;
    }
}
